<?php
show_admin_bar(false);
require_once('TwitterAPIExchange.php');
require_once('wp_bootstrap_navwalker.php');

if ( ! function_exists( 'wager_setup' ) ) {

	function wager_setup() {
		/** Make theme available for translation. */
		load_theme_textdomain( 'wager', get_template_directory() . '/languages' );

		/** Enable support for Post Thumbnails on posts and pages. */
		add_theme_support( 'post-thumbnails' );
		add_image_size( 'blog_thumb', 737, 584, true );
		add_image_size( 'package_thumb', 60, 60, true );
		add_image_size( 'package_featured', 382, 145, true );
		add_image_size( 'package_slide', 282, 400, true );

		/** This theme uses wp_nav_menu() in one location. */
		register_nav_menus( array(
		  'menu-1' => esc_html__( 'Primary menu', 'wager' ),
		  'menu-2' => esc_html__( 'Footer menu', 'wager' )
		) );

		/** Woocommerce Support */
        add_theme_support( 'woocommerce' );
        add_theme_support( 'wc-product-gallery-zoom' );
        add_theme_support( 'wc-product-gallery-lightbox' );
	}
}
add_action( 'after_setup_theme', 'wager_setup' );

/*** Enqueue scripts and styles. */
function wager_scripts() {

	/*** Enqueue styles. */
	wp_enqueue_style('adobe-typekit', 'https://use.typekit.net/udj5bxn.css', array(), false, 'all');
	wp_enqueue_style('plugins', get_template_directory_uri() . '/css/plugins.css', array(), date("ymd-Gis", filemtime( get_template_directory() . '/css/plugins.css' )), 'all');
	wp_enqueue_style( 'wager-style', get_stylesheet_uri(), array(), date("ymd-Gis", filemtime( get_stylesheet_directory())));

	/*** Enqueue scripts. */
	wp_enqueue_script('jquery');
	wp_enqueue_script('plugins', get_template_directory_uri() . '/js/plugins.js', array(), date("ymd-Gis", filemtime( get_template_directory() . '/js/plugins.js' )), true);
	wp_enqueue_script('scripts', get_template_directory_uri() . '/js/scripts.js', array(), date("ymd-Gis", filemtime( get_template_directory() . '/js/scripts.js' )), true);
}
add_action( 'wp_enqueue_scripts', 'wager_scripts' );

/*** Register and enqueue a custom stylesheet in the WordPress admin. */
function admin_scripts() {
	wp_enqueue_style('icon-fonts', get_template_directory_uri() . '/css/icon_fonts.css', array(), false, 'all');
}
add_action( 'admin_enqueue_scripts', 'admin_scripts' );

/** Options Page Header Background */
function wager_admin_dashboard_css() {
	echo '<style type="text/css">
		#acf-group_5a2badeb476ba .hndle { color: #FFF; background: #234FAE; }
		#acf-group_5a2badeb476ba .hndle img { max-width: 186px; margin-right: 15px; }
		#acf-group_5a2badeb476ba .hndle span { display: inline-flex; align-items: center; }
	</style>';
}
add_action('admin_head', 'wager_admin_dashboard_css');

/**
 * Dashboard google map api key support.
 */
add_filter('acf/settings/google_api_key', function () {
	$gmap_api = get_field('google_map_api_key', 'options');
	return $gmap_api;
});

/*** ACF OPTIONS PAGE */
if(function_exists('acf_add_options_page')) {
	acf_add_options_page();
}

/*** Reorder dashboard menu */
function reorder_admin_menu( $__return_true ) {
	return array(
		'index.php',                 // Dashboard
		'separator1',                // --Space--
		'acf-options',               // ACF Theme Settings
		'edit.php?post_type=page',   // Pages 
		'edit.php',                  // Posts
		'separator2',                // --Space--
		'gf_edit_forms',             // Gravity Forms
		'upload.php',                // Media
		'wpseo_dashboard',           // Yoasta
		'gadash_settings',           // Google Analytics
		'themes.php',                // Appearance
		'edit-comments.php',         // Comments 
		'users.php',                 // Users
		'tools.php',                 // Tools
		'options-general.php',       // Settings
		'plugins.php',               // Plugins
	);
}
add_filter( 'custom_menu_order', 'reorder_admin_menu' );
add_filter( 'menu_order', 'reorder_admin_menu' );

/*** Remove dashboard menu */
function remove_admin_menus() {
	// remove_menu_page( 'edit.php' );              // Comments
	remove_menu_page( 'edit-comments.php' );              // Comments
	// remove_menu_page( 'tools.php' );                      // Tools
	// remove_menu_page( 'plugins.php' );                    // Plugings
	remove_menu_page( 'sharethis-general' );          // share this
	// remove_menu_page( 'edit.php?post_type=acf-field-group' ); // Custom Field 
	remove_menu_page( 'pods' );                         // Pods Custom post type
}
add_action( 'admin_menu', 'remove_admin_menus', 999);

/*** GC Color Theme */
function additional_admin_color_schemes() {
	//Get the theme directory
	$theme_dir = get_template_directory_uri();

	//GoingClear
	wp_admin_css_color(
		'goingclear', __('GoingClear'),
		admin_url("css/colors/goingclear/colors.css"),
		array('#8ec652', '#008cc6', '#e5e5e5', '#999'),
		array( 'base' => '#e5f8ff', 'focus' => '#fff', 'current' => '#fff' )
	);
}
add_action('admin_init', 'additional_admin_color_schemes');

/*** Reset GC Color Theme as Default for New Users */
function set_default_admin_color($user_id) {
	$args = array(
		'ID' => $user_id,
		'admin_color' => 'goingclear'
	);
	wp_update_user( $args );
}
add_action('user_register', 'set_default_admin_color');

/*** Remove Login Wiggle */
function remove_login_shake() {
	// remove the wp_shake JavaScript
	remove_action( 'login_head', 'wp_shake_js', 12 );
}
add_action( 'login_head', 'remove_login_shake' );

function custom_header() { 
	echo '<link rel="stylesheet" type="text/css" href="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.css" />'; 
	echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/jquery-3.2.1.min.js"></script>';
	echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.js"></script>';
   
}
add_action('login_head', 'custom_header');

/*** Gravity form user role */
function gforms_editor_access() {
	$role = get_role( 'editor' );
	$role->add_cap( 'gform_full_access' );
}
add_action( 'after_switch_theme', 'gforms_editor_access' );

/*** Get all page id */ 
function getPageID() {
	global $post;
	$postid = $post->ID;
	if( is_home() && get_option( 'page_for_posts' ) ) 
	{
		$postid = get_option( 'page_for_posts' );
	}
	elseif( is_shop() ) 
	{
		$postid = get_option( 'woocommerce_shop_page_id' );
	}
	elseif( is_product_category() ) 
	{
		$postid = get_queried_object()->term_id;
	}
	else 
	{
		$postid = $post->ID;
	}


	return $postid;
}

/*** Limit blog text */
function Limit_Text($text, $limit=30) {
	$array = explode( "\n", wordwrap( $text, $limit));
	return $array['0'];
}

/*** get permalink by template name */
function get_template_link($temp){
	$link = null;
	$pages = get_pages(
		array(
			'meta_key' => '_wp_page_template',
			'meta_value' => $temp
		)
	);
	if(isset($pages[0])){
		$link = get_page_link($pages[0]->ID);
	}
	return $link;
}

/*** Customized header title */
function customizedtitle($title){
	if(is_category() || is_author() || is_date()){
		return get_the_archive_title();
	} else if( is_search()){
	return sprintf( esc_html__( 'Search Results for: %s', 'wager' ), '<span>' . get_search_query() . '</span>' ); 
	}
	return $title;
}

/*** Gravity form anchor */
add_filter( 'gform_confirmation_anchor', '__return_false' );

function form_submit_button($button, $form) {
	return "<button class='btn' id='gform_submit_button_{$form["id"]}'>{$form['button']['text']}</button>";
}
add_filter("gform_submit_button", "form_submit_button", 10, 2);

/*** Featured Image Title Change */
function blogFeaturedImageTitle() {
	remove_meta_box( 'postimagediv', 'post', 'side' );
	add_meta_box( 'postimagediv', __( 'Featured image 737px by 584px', 'wager' ), 'post_thumbnail_meta_box', 'post', 'side' );
}
add_action('do_meta_boxes', 'blogFeaturedImageTitle' );

/*** ACF Button Function */
if( class_exists('acf') ) { 
    function acfButton($field, $style = null, $up = null) {
        if ($field['button']['text']) 
        {
            if ($field['button']['type'] == 'internal') 
            {
                printf('<a href="%s" class="btn %s %s">%s</a>', esc_url($field['button']['internal_url']), $style, $up, $field['button']['text']);
            }
            elseif ($field['button']['type'] == 'external')
            {
                printf('<a href="%s" class="btn %s %s" target="_blank">%s</a>', esc_url($field['button']['external_url']), $style, $up, $field['button']['text']);
            }
        }
    }
}

/*** Breadcrumb */
function wager_breadcrumb() {

	$delimiter = '<span class="angle-right">/</span>';
	$home = 'Home'; 
	$before = '<span class="current-page">'; 
	$after = '</span>'; 

	$events = get_post_type_object('tribe_events');
   
	if ( !is_front_page() ) {
   
	echo '<nav class="breadcrumb">';
   
	global $post;
	$homeLink = get_bloginfo('url');
	$blogTitle = get_the_title( get_option( 'page_for_posts' ) );
	$blogLink = get_permalink( get_option( 'page_for_posts' ) );
	echo '<a href="' . home_url() . '">' . $home . '</a> ' . $delimiter . ' ';
   
	if ( is_home() ) {

		echo $before . ' ' . $blogTitle . ' ' . $after;

	} elseif ( is_category() ) {

		global $wp_query;
		$cat_obj = $wp_query->get_queried_object();
		$thisCat = $cat_obj->term_id;
		$thisCat = get_category($thisCat);
		$parentCat = get_category($thisCat->parent);
		if ($thisCat->parent != 0) echo(get_category_parents($parentCat, TRUE, ' ' . $delimiter . ' '));
		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo $before . single_cat_title('', false) . $after;

	} elseif ( is_day() ) {
		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo '<a href="' . get_year_link(get_the_time('Y')) . '">' . get_the_time('Y') . '</a> ' . $delimiter . ' ';
		echo '<a href="' . get_month_link(get_the_time('Y'),get_the_time('m')) . '">' . get_the_time('F') . '</a> ' . $delimiter . ' ';
		echo $before . get_the_time('d') . $after;
   
	} elseif ( is_month() ) {

		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo '<a href="' . get_year_link(get_the_time('Y')) . '">' . get_the_time('Y') . '</a> ' . $delimiter . ' ';
		echo $before . get_the_time('F') . $after;
   
	} elseif ( is_year() ) {

		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo $before . get_the_time('Y') . $after;
   
	} else if ( is_author() ) {
		global $author;
		$userdata = get_userdata( $author );
			
		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo $before . ' ' . $userdata->display_name . ' ' . $after;
		   
	  } elseif ( is_single() && !is_attachment() ) {

		global $wp_query;
		$cat_obj = $wp_query->get_queried_object();

		if ($cat_obj->post_type === 'tribe_events') {

			echo '<a href="' . $homeLink . '/' . $events->rewrite['slug'] . '/">'.$events->rewrite['slug'].'</a> ' . $delimiter . ' ';
			echo $before . $cat_obj->post_title . $after;

		} elseif ( get_post_type() != 'post' ) {

			$post_type = get_post_type_object(get_post_type());
			$slug = $post_type->rewrite;
			echo '<a href="' . $homeLink . '/' . $slug['slug'] . '/">' . $post_type->labels->singular_name . '</a> ' . $delimiter . ' ';
			echo $before . get_the_title() . $after;

		} else {

			$cat = get_the_category(); $cat = $cat[0];
			echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
			//echo get_category_parents($cat, TRUE, ' ' . $delimiter . ' ');
			echo $before . get_the_title() . $after;

		}
	} elseif ( !is_single() && !is_page() && get_post_type() != 'post' && !is_404() && !is_search() ) {
		$post_type = get_post_type_object(get_post_type());
	
		if ($events) {
			echo $before . $events->rewrite['slug'] . $after;
		} else {
			echo $before . $post_type->labels->singular_name . $after;
		}
   
	} elseif ( is_attachment() ) {

		$parent = get_post($post->post_parent);
		$cat = get_the_category($parent->ID); $cat = $cat[0];
		echo get_category_parents($cat, TRUE, ' ' . $delimiter . ' ');
		echo '<a href="' . get_permalink($parent) . '">' . $parent->post_title . '</a> ' . $delimiter . ' ';
		echo $before . get_the_title() . $after;
   
	} elseif ( is_page() && !$post->post_parent ) {

		echo $before . get_the_title() . $after;

	} elseif ( is_page() && $post->post_parent ) {

		$parent_id = $post->post_parent;
		$breadcrumbs = array();
		while ($parent_id) {
			$page = get_page($parent_id);
			$breadcrumbs[] = '<a href="' . get_permalink($page->ID) . '">' . get_the_title($page->ID) . '</a>';
			$parent_id = $page->post_parent;
		}

		$breadcrumbs = array_reverse($breadcrumbs);
		foreach ($breadcrumbs as $crumb) echo $crumb . ' ' . $delimiter . ' ';
			echo $before . get_the_title() . $after;
   
	} elseif ( is_search() ) {

		echo $before . ' ' . $blogTitle . ' ' . $after;
		echo $before . 'Search Results for: "' . get_search_query() . '"' . $after;

	} elseif ( is_tag() ) {
		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo $before . 'Posts with the tag "' . single_tag_title('', false) . '"' . $after;

	} elseif ( is_tag() ) {

		echo '<a href="' . $blogLink . '">'.$blogTitle.'</a> ' . $delimiter . ' ';
		echo $before . 'Posts with the tag "' . single_tag_title('', false) . '"' . $after;

	} elseif ( is_404() ) {

		echo $before . 'Error 404' . $after;
	}
   
	if ( get_query_var('paged') ) {
		if ( is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author() ) echo ' (';
		  echo ': ' . __('Page') . ' ' . get_query_var('paged');
		if ( is_category() || is_day() || is_month() || is_year() || is_search() || is_tag() || is_author() ) echo ')';
	}
	echo '</nav>';
	} 
}

/*** Twitter Feeds */
class twitterFeeds {
    private $screen_name;
    private $user_id;
    private $limit;
    private $settings;
    public function __construct(){
        $this->setApiInfo();
    }
    private function setApiInfo() {
      $twitter_feeds = get_field('twitter_feeds', 'options');
        $this->settings = array(
            'oauth_access_token'        => $twitter_feeds['access_token'],
            'oauth_access_token_secret' => $twitter_feeds['access_token_secret'],
            'consumer_key'              => $twitter_feeds['consumer_key'],
            'consumer_secret'           => $twitter_feeds['consumer_secret'],
        );
        $this->screen_name = $twitter_feeds['username'];
        $this->user_id = $twitter_feeds['userid'];
        $this->limit = $twitter_feeds['tweet_limit'] ? $twitter_feeds['tweet_limit'] : 2;
    }
    private function fetchRecentTweents(){
        $url = 'https://api.twitter.com/1.1/statuses/user_timeline.json';
        $getfield = '?screen_name=' . $this->screen_name . '&count=' . $this->limit;
        $requestMethod = 'GET';
        $twitter = new TwitterAPIExchange($this->settings);
        $query = $twitter->setGetfield($getfield)
                     ->buildOauth($url, $requestMethod)
                     ->performRequest(true, array(CURLOPT_CAINFO => dirname(__FILE__) . '/cacert.pem'));
        
        $timelines = json_decode($query);
        // return the last tweet
        return $timelines;
    }
    private function fetchTwitterProfile(){
        $url = 'https://api.twitter.com/1.1/users/show.json';
        $getfield = '?screen_name=' . $this->screen_name . '&user_id=' . $this->user_id;
        $requestMethod = 'GET';
        $twitter = new TwitterAPIExchange($this->settings);
        $query = $twitter->setGetfield($getfield)
                     ->buildOauth($url, $requestMethod)
                     ->performRequest(true, array(CURLOPT_CAINFO => dirname(__FILE__) . '/cacert.pem'));
        $profile = json_decode($query);
        // return the profile information
        return $profile;
    }
    public function getTwitterFeeds(){
        $profile = $this->fetchTwitterProfile();
        $tweets = $this->fetchRecentTweents();
        $output = '';
        foreach ($tweets as $tweet) {
        $output .= '<div class="item">
                        <a href="//twitter.com/'. $profile->screen_name .'/status/'.$tweet->id.'" target="_blank"><p>'. $tweet->text .'</p></a>
                        <div class="twitter-feed-footer">
                            <div class="media">
                                <img src="'.$profile->profile_image_url.'" class="img-responsive" alt="'. $profile->screen_name .'">
                            </div>
                            <div class="text">
                                <h4 class="name">'. $profile->name .'</h4>
                                <a href="//twitter.com/'. $profile->screen_name .'" class="username" target="_blank">@'. $profile->screen_name .'</a>
                            </div>
                        </div>
                    </div>';
        }
        return $output;
    }
    private function formatTweetTime( $time ) {
        // Get current timestamp.
        $now = strtotime( 'now' );
     
        // Get timestamp when tweet created.
        $created = strtotime( $time );
     
        // Get difference.
        $difference = $now - $created;
     
        // Calculate different time values.
        $minute = 60;
        $hour = $minute * 60;
        $day = $hour * 24;
        $week = $day * 7;
     
        if ( is_numeric( $difference ) && $difference > 0 ) {
     
            // If less than 3 seconds.
            if ( $difference < 3 ) {
                return __( 'Just now', 'twitter_tweets_widget' );
            }
     
            // If less than minute.
            if ( $difference < $minute ) {
                return floor( $difference ) . ' ' . __( 'seconds ago', 'twitter_tweets_widget' );;
            }
     
            // If less than 2 minutes.
            if ( $difference < $minute * 2 ) {
                return __( '1 minute ago', 'twitter_tweets_widget' );
            }
     
            // If less than hour.
            if ( $difference < $hour ) {
                return floor( $difference / $minute ) . ' ' . __( 'minutes ago', 'twitter_tweets_widget' );
            }
     
            // If less than 2 hours.
            if ( $difference < $hour * 2 ) {
                return __( '1 hour ago', 'twitter_tweets_widget' );
            }
     
            // If less than day.
            if ( $difference < $day ) {
                return floor( $difference / $hour ) . ' ' . __( 'hours ago', 'twitter_tweets_widget' );
            }
     
            // If more than day, but less than 2 days.
            if ( $difference > $day && $difference < $day * 2 ) {
                return __( 'yesterday', 'twitter_tweets_widget' );;
            }
     
            // If less than year.
            if ( $difference < $day * 365 ) {
                return floor( $difference / $day ) . ' ' . __( 'days ago', 'twitter_tweets_widget' );
            }
     
            // Else return more than a year.
            return __( 'over a year ago', 'twitter_tweets_widget' );
        }
    }
}

/**
 * Extend WordPress search to include custom fields
 * http://adambalee.com
 *
 * Join posts and postmeta tables
 * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_join
 */
function phepmrc_acf_search_join( $join ) {
	global $wpdb;
	if ( is_search() ) {    
		$join .=' LEFT JOIN '.$wpdb->postmeta. ' ON '. $wpdb->posts . '.ID = ' . $wpdb->postmeta . '.post_id ';
	}
	return $join;
}
add_filter('posts_join', 'phepmrc_acf_search_join' );

/**
 * Modify the search query with posts_where
 * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_where
 */
function phepmrc_acf_search_where( $where ) {
	global $pagenow, $wpdb;
	if ( is_search() ) {
		$where = preg_replace(
			"/\(\s*".$wpdb->posts.".post_title\s+LIKE\s*(\'[^\']+\')\s*\)/",
			"(".$wpdb->posts.".post_title LIKE $1) OR (".$wpdb->postmeta.".meta_value LIKE $1)", $where );
	}
	return $where;
}
add_filter( 'posts_where', 'phepmrc_acf_search_where' );

/**
 * Prevent duplicates
 * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_distinct
 */
function phepmrc_acf_search_distinct( $where ) {
	global $wpdb;
	if ( is_search() ) {
		return "DISTINCT";
	}
	return $where;
}
add_filter( 'posts_distinct', 'phepmrc_acf_search_distinct' );

/*** Popular Posts */
function count_post_visits() {
	if( is_single() ) {
		global $post;
		$views = get_post_meta( $post->ID, 'my_post_viewed', true );
		if( $views == '' ) {
			update_post_meta( $post->ID, 'my_post_viewed', '1' );	
		} else {
			$views_no = intval( $views );
			update_post_meta( $post->ID, 'my_post_viewed', ++$views_no );
		}
	}
}
add_action( 'wp_head', 'count_post_visits' );


/*** Basic Content */
function basic_content( $group_data ) {    
	echo '<div class="basic-content-wrapper">';
	echo $group_data['contnet'];
	echo '</div>';

	$add_button = $group_data['add_button'];

	if( $add_button ):
		$button_setting   = $group_data['button_setting'];
		$external_button  = $button_setting['external_button'];
		$link       = '';
		if( $external_button )
		{ 
			$link = $button_setting['url']; 
		} 
		else 
		{ 
			$link = $button_setting['page_link']; 
		}
		?>
		<a href="<?php echo $link; ?>" target="<?php echo ( $external_button )?'_blank':''; ?>" class="btn text-uppercase">
			<?php echo $button_setting['title']; ?>       
		</a>

	<?php
	endif; // button setting

	//check Form
	$add_form     = $group_data['add_form'];
	echo '<div class="basic-form">';
	if( $add_form ):
		$form_title = $group_data['form_title'];
		$form_id  = $group_data['select_form'];
		echo ( $form_title )?'<h3>'.$form_title.'</h3>':'';
		echo do_shortcode('[gravityform id='.$form_id.' title=false description=false ajax=true tabindex=49]');
	endif;
	echo '</div>';
}

/***********************************
* Woocommerce 
************************************/
/** Woocommerce Remove Action */
remove_action( 'woocommerce_after_shop_loop', 'woocommerce_pagination', 10 );
remove_action( 'woocommerce_before_shop_loop', 'woocommerce_result_count', 20 );
remove_action( 'woocommerce_before_shop_loop', 'woocommerce_catalog_ordering', 30 );
remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_price', 10 );

add_action( 'woocommerce_after_add_to_cart_quantity', 'woocommerce_template_single_price', 9 );

/** Woocommerce Add Action */
/* top sorting bar */
add_action('woocommerce_before_shop_loop', 'warp_before_filter_items', 10);
add_action( 'woocommerce_before_shop_loop', 'woocommerce_catalog_ordering', 11 );
add_action( 'woocommerce_before_shop_loop', 'result_pagination_wrapper', 12 );
add_action( 'woocommerce_before_shop_loop', 'woocommerce_result_count', 13 );
add_action( 'woocommerce_before_shop_loop', 'woocommerce_pagination', 14 );
add_action('woocommerce_before_shop_loop', 'end_warp_before_filter_items', 15);
/* bottom sorting bar */
add_action('woocommerce_after_shop_loop', 'warp_before_filter_items', 10);
add_action( 'woocommerce_after_shop_loop', 'woocommerce_catalog_ordering', 11 );
add_action( 'woocommerce_after_shop_loop', 'result_pagination_wrapper', 12 );
add_action( 'woocommerce_after_shop_loop', 'woocommerce_result_count', 13 );
add_action( 'woocommerce_after_shop_loop', 'woocommerce_pagination', 14 );
add_action('woocommerce_after_shop_loop', 'end_warp_before_filter_items', 15);
/** Remove the product rating display on product loops */ 
remove_action( 'woocommerce_after_shop_loop_item_title', 'woocommerce_template_loop_rating', 5 );
/** Woocommerce Shop After product filter wrapper class add */
function result_pagination_wrapper(){
  echo '<div class="pagination_result_count">';
}
function warp_before_filter_items() {
  echo '<div class="wrap_filter_items d-flex flex-wrap justify-content-between">';
}
function end_warp_before_filter_items() {
  echo '</div></div>';
}

function woo_before_add_to_cart_quantity() {
  echo '<div class="price-button d-flex flex-wrap align-items-center justify-content-between">';
}
add_action( 'woocommerce_before_add_to_cart_quantity', 'woo_before_add_to_cart_quantity', 10 );

function woo_after_add_to_cart_button() {
  echo '</div>';
}
add_action( 'woocommerce_after_add_to_cart_button', 'woo_after_add_to_cart_button', 50 );

/** Shop page how many products show */
add_filter( 'loop_shop_per_page', create_function( '$cols', 'return 9;' ), 20 );

/** Header right mini cart number ajaxify */
function wager_header_ajaxify_add_to_cart( $fragments ) {
  global $woocommerce;
  ob_start();

  ?>
  <a href="<?php echo $woocommerce->cart->get_cart_url(); ?>" title="<?php _e('View your shopping cart', 'wager'); ?>" class="ajaxify_cart">
        <i class="icon-cart"></i>
        <span class="cart_quantity <?php if('0' == WC()->cart->get_cart_contents_count()) echo 'hidden'; ?>"><?php echo sprintf(_n('%d', '%d', $woocommerce->cart->cart_contents_count, 'wager'), $woocommerce->cart->cart_contents_count);?></span>
  </a>
  <?php
  
  $fragments['.ajaxify_cart'] = ob_get_clean();
  
  return $fragments;
  
}
add_filter('add_to_cart_fragments', 'wager_header_ajaxify_add_to_cart');
/** Header right mini cart hover load cart item ajaxify * Js Part settings.js File */
function mode_theme_update_mini_cart() {
    echo wc_get_template( 'cart/mini-cart.php' );
    die();
}
add_filter( 'wp_ajax_nopriv_mode_theme_update_mini_cart', 'mode_theme_update_mini_cart' );
add_filter( 'wp_ajax_mode_theme_update_mini_cart', 'mode_theme_update_mini_cart' );

/*** Woocommerce Single Page */
remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_meta', 40 );
remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20 );

// Upsell Product remove single page
remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_upsell_display', 15 );


function woo_remove_product_tabs( $tabs ) {
    unset( $tabs['description'] );          // Remove the description tab
    unset( $tabs['reviews'] );          // Remove the reviews tab
    unset( $tabs['additional_information'] );   // Remove the additional information tab
    return $tabs;
}
add_filter( 'woocommerce_product_tabs', 'woo_remove_product_tabs', 98 );


/** Change checkout form field placeholder and label */  
function wager_checkout_fields_placeholder( $fields ) {
    // Label remove
    $fields['billing']['billing_first_name']['label'] = '';
    $fields['billing']['billing_last_name']['label'] = '';
    $fields['billing']['billing_company']['label'] = '';
    $fields['billing']['billing_email']['label'] = '';
    $fields['billing']['billing_phone']['label'] = '';
    $fields['billing']['billing_postcode']['label'] = '';
    $fields['billing']['billing_country']['label'] = '';
    $fields['billing']['billing_address_1']['label'] = '';
    // Placeholder add 
    $fields['billing']['billing_first_name']['placeholder'] = 'First Name *';
    $fields['billing']['billing_last_name']['placeholder'] = 'Last Name *';
    $fields['billing']['billing_company']['placeholder'] = 'Company Name';
    $fields['billing']['billing_email']['placeholder'] = 'Email Address *';
    $fields['billing']['billing_phone']['placeholder'] = 'Phone *';
    $fields['billing']['billing_postcode'] = array(
        'placeholder' => _x('Postcode *', 'placeholder', 'woocommerce'),
    );
    $fields['billing']['billing_address_1'] = array(
        'type' => 'textarea',
        'placeholder' => _x('Address *', 'placeholder', 'woocommerce'),
    );
    // Add field priority
    $fields['billing']['billing_email']['priority'] = 30;
    $fields['billing']['billing_phone']['priority'] = 40;
    $fields['billing']['billing_country']['priority'] = 50;
    $fields['billing']['billing_postcode']['priority'] = 60;
    $fields['billing']['billing_address_1']['priority'] = 90;
    //$fields['billing']['billing_state']['label'] = '';
    return $fields;
}
add_filter( 'woocommerce_checkout_fields' , 'wager_checkout_fields_placeholder', 10, 1);

/** Checkout Page Payment Method heading Add */
function payment_method_heading() {
    echo '<h3 id="payment_method_heading">Payment Method</h3>';
}
add_action( 'woocommerce_review_order_before_payment', 'payment_method_heading' );

function change_shipping_text($sprintf, $i, $package){
    return 'Shipping & Handling';
}
add_filter('woocommerce_shipping_package_name', 'change_shipping_text', 10, 3);


/*** add SVG to allowed file uploads */
function wager_custom_mime_types( $mimes ) {
	// New allowed mime types.
	$mimes['svg'] = 'image/svg+xml';
	$mimes['svgz'] = 'image/svg+xml';
	$mimes['doc'] = 'application/msword';
	 
	// Optional. Remove a mime type.
	unset( $mimes['exe'] );
	 
	return $mimes;
}
add_filter( 'upload_mimes', 'wager_custom_mime_types' );